@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //*************************************************************************
 //
-// Copyright 2000-2009 by Wilson Snyder.  This program is free software;
+// Copyright 2000-2010 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 //
@@ -69,7 +69,7 @@ public:  // But for internalish use only
     int lexToBison(VParseBisonYYSType* yylvalp);
     bool eofToLex() const { return m_eof && m_buffers.empty(); }
     bool inCellDefine() const;
-    int inputToLex(char* buf, int max_size);
+    size_t inputToLex(char* buf, size_t max_size);
 
     // Symbol table
     VSymStack&	syms() { return m_syms; }
@@ -135,7 +135,7 @@ public:
     string unreadback() const { return (m_useUnreadback ? m_unreadback : "new(...,use_unreadback=>0) was used"); }
     void unreadback(const string& text) { if (m_useUnreadback) m_unreadback = text; }
     void unreadbackCat(const string& text) { if (m_useUnreadback) m_unreadback += text; }
-    void unreadbackCat(const char* textp, int len) { unreadbackCat(string(textp,len)); }
+    void unreadbackCat(const char* textp, size_t len) { unreadbackCat(string(textp,len)); }
 
     // The default behavior is to pass all unknown `defines right through.
     // This lets the user determine how to report the errors.  It also nicely
@@ -164,6 +164,7 @@ public:
     virtual void contassignCb(VFileLine* fl, const string& kwd, const string& lhs, const string& rhs) = 0;
     virtual void endcellCb(VFileLine* fl, const string& kwd) = 0;
     virtual void endinterfaceCb(VFileLine* fl, const string& kwd) = 0;
+    virtual void endmodportCb(VFileLine* fl, const string& kwd) = 0;
     virtual void endmoduleCb(VFileLine* fl, const string& kwd) = 0;
     virtual void endpackageCb(VFileLine* fl, const string& kwd) = 0;
     virtual void endprogramCb(VFileLine* fl, const string& kwd) = 0;
@@ -172,6 +173,7 @@ public:
     virtual void importCb(VFileLine* fl, const string& package, const string& id) = 0;
     virtual void instantCb(VFileLine* fl, const string& mod, const string& cell, const string& range) = 0;
     virtual void interfaceCb(VFileLine* fl, const string& kwd, const string& name) = 0;
+    virtual void modportCb(VFileLine* fl, const string& kwd, const string& name) = 0;
     virtual void moduleCb(VFileLine* fl, const string& kwd, const string& name, bool, bool celldefine) = 0;
     virtual void packageCb(VFileLine* fl, const string& kwd, const string& name) = 0;
     virtual void parampinCb(VFileLine* fl, const string& name, const string& conn, int index) = 0;
@@ -185,8 +187,8 @@ public:
     // CALLBACKGEN_GENERATED_END - GENERATED AUTOMATICALLY by callbackgen
 
     // UTILITIES
-    void error(string msg) { inFilelinep()->error(msg); }	///< Report a error
-    void fatal(string msg) { inFilelinep()->fatal(msg); }	///< Report a fatal error
+    void error(const string& msg) { inFilelinep()->error(msg); }	///< Report a error
+    void fatal(const string& msg) { inFilelinep()->fatal(msg); }	///< Report a fatal error
 };
 
 #endif // Guard
